from CvPythonExtensions import *
import PyHelpers
import Spreads
import Damages
import DisasterUtils
import csv
gc = CyGlobalContext()




class Disasters:
    
    def __init__(self):
        
        ## Fires
        self.iMinFireLife = 2 ## minimum turns a fire burns
        self.iBurnoutChance = 75 ## decrease this to have randomly long lasting fires
        self.iBaseSpreadChance = 10 ## base chance that a fire will spread to adjacent tile
        self.lFeatures = [] ## features that modify fire chance, set in updateFires
        self.lFeatureWeights = [70,60] ## the chances
        self.lTerrains = [] ## terrains that modify fire chance, set in updateFires
        self.lTerrainWeights = [-10,-10] 
        self.lImprovements = [] ## improvements that modify fire chance, set in updateFires
        self.lImprovementWeights = [10,15,25,35,20]
        self.iCityWeight = 40 ## chance to spread to city
        self.iBurnBuildingChance = 10 ## chance to burn a building while in city
        self.iFireBurnPeople = 5 ## maximum number of people that die in a fire in a city in one turn
        self.iBurnImprovementChance = 75
        self.iBurnForestChance = 90
        self.counter = 0 ## this is used for the graphical effect don't touch it
        self.eachTurnForestFireChance = 10 ## this is out of 100. chance that forest fire starts anywhere
        self.iPillageFireChance = 25 ## chance that a pillaging unit will set fire to the tile
        self.iRazeCityFireChance = 75 ## chance that razing a city sets fire to the tile
        
    def updateFires(self):
        self.lFeatures = [gc.getInfoTypeForString("FEATURE_FOREST"),gc.getInfoTypeForString("FEATURE_JUNGLE")]
        self.lTerrains = [gc.getInfoTypeForString("TERRAIN_DESERT"),gc.getInfoTypeForString("TERRAIN_PEAK")]
        self.lImprovements = [gc.getInfoTypeForString("IMPROVEMENT_COTTAGE"),gc.getInfoTypeForString("IMPROVEMENT_HAMLET"),gc.getInfoTypeForString("IMPROVEMENT_VILAGE"),gc.getInfoTypeForString("IMPROVEMENT_TOWN"),gc.getInfoTypeForString("IMPROVEMENT_FORT")]
        
        for pFire in DisasterUtils.lFires:
            pFire.iTurnsAlive += 1
            if pFire.iTurnsAlive > self.iMinFireLife:
                iBurn = CyGame().getSorenRandNum(100,"burns out")
                if iBurn < self.iBurnoutChance:
                    self.removeFire(pFire)
                
                newfires = Spreads.Spreads().spreadNear(pFire.X,pFire.Y,1,self.iBaseSpreadChance,False,self.iCityWeight,self.lFeatures,self.lFeatureWeights,self.lImprovements,self.lImprovementWeights,self.lTerrains,self.lTerrainWeights)
                for pTile in newfires:
                    self.setFire(pTile.getX(),pTile.getY())
                pFirePlot = gc.getMap().plot(pFire.X,pFire.Y)
                if pFirePlot.isCity():
                    Damages.Damages().destroyBuildings(pFirePlot.getPlotCity(), True, [], self.iBurnBuildingChance, -1, True, True)
                    ## message
                    ## Damages.Damages().damageUnits(pFirePlot, -1, True)
                    ## message
                    Damages.Damages().killPopulation(pFirePlot.getPlotCity(), self.iFireBurnPeople, True)
                    ## message
                if pFirePlot.getNumUnits() > 0:
                    for p in range(pFirePlot.getNumUnits()):
                        unitunit = pFirePlot.getUnit(p)
                        if unitunit.isHuman():
                            continue
                        unitunit.setXY(unitunit.getX()+3,unitunit.getY()+CyGame().getSorenRandNum(4,"dist"))
                        ## this should teleport AI units a bit away
                    Damages.Damages().damageUnits(pFirePlot)
                
                
    def setFire(self, iX, iY):
        if self.isFire(iX, iY):
             return
        else:
            newFire = Fire(iX, iY)
            DisasterUtils.lFires.append(newFire)
        
    def isFire(self, iX, iY):
        for pFire in DisasterUtils.lFires:
            if pFire.X == iX and pFire.Y == iY:
                return True
        return False
    
    def removeFire(self, pFire):
        DisasterUtils.lFires.remove(pFire)
        pPlot = gc.getMap().plot(pFire.X,pFire.Y)
        Damages.Damages().damageTile(pPlot,self.iBurnImprovementChance,self.iBurnForestChance)
        del pFire
        ## remove fire marker unit from the plot
        if pPlot.getNumUnits() > 0:
            for i in range(pPlot.getNumUnits()):
                pUnit = pPlot.getUnit(i)
                if pUnit.getUnitType() == gc.getInfoTypeForString("UNIT_FIREMARKER"):
                    pUnit.kill(False, pBarb)
        
    def updateFireGraphics(self):
        self.counter += 1
        if self.counter % 35 == 0 or self.counter < 5:
            for pFire in DisasterUtils.lFires:
                CyEngine().triggerEffect(gc.getInfoTypeForString("EFFECT_DISASTER_FIRE"), gc.getMap().plot(pFire.X,pFire.Y).getPoint())
        if self.counter > 1000:
            self.counter = 0
            
    def tryForestFire(self):
        randFire = CyGame().getSorenRandNum(100,"fire chance")
        if randFire < self.eachTurnForestFireChance:
            bForest = False
            while not bForest:
                randx = CyGame().getSorenRandNum(gc.getMap().getGridWidth(),"random x")
                randy = CyGame().getSorenRandNum(gc.getMap().getGridHeight(),"random y")
                if gc.getMap().plot(randx, randy).getFeatureType() == gc.getInfoTypeForString("FEATURE_FOREST"):
                    bForest = True
            self.setFire(randx, randy)
    
    def tryRazeCityFire(self, iX, iY):
        randFire = CyGame().getSorenRandNum(100,"fire chance")
        if randFire < self.iRazeCityFireChance:
            self.setFire(iX, iY)
    
    def tryPillageFire(self, iX, iY):
        randFire = CyGame().getSorenRandNum(100,"fire chance")
        if randFire < self.iPillageFireChance:
            self.setFire(iX, iY)    
            
    def saveFires(self, file):
        attributes = 3
        lWriteList = []
        for pFire in DisasterUtils.lFires:
            for count in range(attributes):
                if count % 3 == 0:
                    lWriteList.append(str(pFire.X))
                    lWriteList.append("\n")
                elif count % 3 == 1:
                    lWriteList.append(str(pFire.Y))
                    lWriteList.append("\n")
                elif count % 3 == 2:
                    lWriteList.append(str(pFire.iTurnsAlive))
                    lWriteList.append("\n")
        file.writelines(lWriteList)
        
    def loadFires(self, file):
        count = 0
        xx = 0
        yy = 0
        tt = 0        
        for line in file.readlines():
            if count % 3 == 0:
                xx = int(line)
            elif count % 3 == 1:
                yy = int(line)
            elif count % 3 == 2:
                tt = int(line)
                pFire = Fire(xx, yy, tt)
                DisasterUtils.lFires.append(pFire)
                xx = 0
                yy = 0
                tt = 0
            count += 1
            
        
class Fire:
    def __init__(self, iX, iY, TurnsAlive = 0):
        self.X = iX
        self.Y = iY
        self.iTurnsAlive = 0
        if TurnsAlive != 0:
            self.iTurnsAlive = TurnsAlive
        
    
        